using UnityEngine;
using System;
using System.Collections.Generic;
using static MacGruber.Breathing;
using static MacGruber.Utils;
using SimpleJSON;

namespace MacGruber
{
	public class BreathingDebug : MVRScript
	{
		private Breathing myBreathing;

		private JSONStorableStringChooser myDataset;
		private JSONStorableFloat myEntry;
		private JSONStorableString myEntryFile;
		private JSONStorableString myNowPlayingFile;

		public override void Init()
		{
			myBreathing = FindWithinSamePlugin<Breathing>(this);
			if (myBreathing == null)
				return;

			InitUI();
		}

		private void OnDestroy()
        {
			Utils.OnDestroyUI();
        }

		private void InitUI()
		{
			Utils.OnInitUI(CreateUIElement);
			Utils.SetupInfoTextNoScroll(this,
				"<color=#606060><size=40><b>Breathing Debug</b></size>\nPlay specific breath audio for debug purposes. Does only work with debug database.</color>\n\n" +
				"<b>Debug Reload Database:</b> Reload database from local file instead of the AssetBundle. Use this for quicker iteration time when tweaking numbers. Path is hardcoded:\n<i>Custom/Scripts/MacGruber/Life/\nDatabase.json</i>\n\n",
				630.0f, true
			);

			Utils.SetupButton(this, "Debug Reload Database", ReloadDatabase, false);

			myDataset = new JSONStorableStringChooser("Dataset", new List<string>(), string.Empty, "Dataset");
			myDataset.storeType = JSONStorableParam.StoreType.Full;
			CreateScrollablePopup(myDataset, false);
			RegisterStringChooser(myDataset);
			myDataset.setCallbackFunction  += (string v) => {
				UpdateDataset();
			};

			myEntry = SetupSliderFloat(this, "Entry", 1, 1, 1, false);
			myEntry.setCallbackFunction  += (float v) => {
				int i = Mathf.RoundToInt(v);
				myEntry.valNoCallback = i;

				Breathing.Dataset dataset;
				if (myBreathing.Datasets.TryGetValue(myDataset.val, out dataset))
				{
					if (i >= 1 && i <= dataset.myEntries.Count)
					{
						myEntryFile.val = dataset.myEntries[i-1].Name;
						return;
					}
				}

				myEntryFile.val = "";
			};

			myEntryFile = SetupInfoOneLine(this, "", false);

			Utils.SetupButton(this, "Debug Queue Entry", QueueEntry, false);

			CreateSpacer(false);

			SetupInfoOneLine(this, "Now playing:", false);
			myNowPlayingFile = SetupInfoOneLine(this, "", false);
        }

		private void ReloadDatabase()
		{
			myBreathing.DebugReloadDatabase();

			UpdateDatabase();
		}

		private void UpdateDatabase()
		{
			List<string> datasetNames = myBreathing.Datasets.Keys.ToList();
			myDataset.choices = datasetNames;
			if (datasetNames.Count > 0 && !datasetNames.Contains(myDataset.val))
				myDataset.val = datasetNames[0];

			UpdateDataset();
		}

		private void UpdateDataset()
		{
			Breathing.Dataset dataset;
			if (myBreathing.Datasets.TryGetValue(myDataset.val, out dataset))
			{
				myEntry.max = dataset.myEntries.Count;
				myEntry.setCallbackFunction(Mathf.Clamp(myEntry.val, 1, myEntry.max));
			}
			else
			{
				myEntry.max = 1;
				myEntry.setCallbackFunction(1);
			}
		}

		private void QueueEntry()
		{
			int i = Mathf.RoundToInt(myEntry.val) - 1;
			myBreathing.QueueDebugEntry(myDataset.val, i);
		}

		private void Update()
		{
			Breath breath = myBreathing.Get(CURR);
			if (breath.IsValid())
				myNowPlayingFile.val = breath.Entry.Name;
			else
				myNowPlayingFile.val = "...";
		}
	}

	public partial class Breathing : MVRScript
	{
		////////////////////////////////////////////////////////////////////////////////////
		// Debug Extensions

		internal void DebugReloadDatabase()
		{
			JSONNode database = SuperController.singleton.LoadJSON("Custom/Scripts/MacGruber/Life/Database.json");
			LoadDatabase(database);
		}

		internal Dictionary<string, Dataset> Datasets { get { return myDatasets; } }

		internal void QueueDebugEntry(string aDataset, int aEntry)
		{
			myQueueType = TYPE_DEBUG;
			myQueueDebugDataset = aDataset;
			myQueueDebugEntry = aEntry;
		}
	}
}